package scanner;

import java.io.InputStream;
import java.io.FilterInputStream;
import java.io.IOException;

/**
 * Removes Lisp comments from an input stream.
 */
public class LispFilterInputStream extends FilterInputStream {

    private boolean isInQuotedString;
    private int previousCharacter;
    private int nextCharacter;

    public LispFilterInputStream(InputStream underlyingInputStream) {
        super(underlyingInputStream);

        isInQuotedString = false;
        previousCharacter = 0;
        nextCharacter = 0;
    }

    @Override
    public int read() throws IOException {
        readNextCharacter();

        if (haveEnteredComment())
            consumeAllBytesInComment();

        return nextCharacter;
    }

    private void readNextCharacter() throws IOException {
        previousCharacter = nextCharacter;
        nextCharacter = super.read();

        indicateEncounterWithStringBoundary();
    }

    private void indicateEncounterWithStringBoundary() {
        if (haveEncounteredStringBoundary())
            isInQuotedString = !isInQuotedString;
    }

    private boolean haveEncounteredStringBoundary() {
        return (previousCharacter != '\\') && (nextCharacter == '\"');
    }

    private boolean haveEnteredComment() {
        return (nextCharacter == ';') && (!isInQuotedString);
    }

    private void consumeAllBytesInComment() throws IOException {
        while (stillInComment())
            nextCharacter = super.read();
    }

    private boolean stillInComment() {
        return (nextCharacter != '\n') && (nextCharacter != -1);
    }

}
